@@ -9,6 +9,7 @@ from Caches        import *
 from cachetruster  import *
 from PackageName   import *
 from suffix        import *
+from SystemEnviros import *
 
 def q_usePackageRoot(p):
 	d = dir(p)
@@ -124,14 +125,15 @@ def isAlreadyInstalled2(aeo):
 	else:        return ae
 	
 def nullPackage():
-	p = Package('','','','','','',{},[],  [],[],[],[],[],[],[],[],'',[],1,[])
+#	p = Package('','','','','','',{},[],  [],[],[],[],[],[],[],[],'',[],1,[])
+	p = Package('','','','','','',{},[],  [],[],[],[],[],[],[],[],'',[],1,1,[],[],{})
 	return p
 
 class Package:
 	'Represents a fetchable, installable, removable package.'
 	def __init__(self,name,description,url,doc,cache,source,systems,depends,\
 	             exists,inpath,bins,paths,enviros,daemons,inst,setup,demo,\
-		     already,suffix,usePackageRoot,uninstall,download):
+		     already,suffix,usePackageRoot,uninstall,reqs,sys_setenv,pubEnviros,download):
 		self.name           = name
 		self.description    = description
 		self.url            = url
@@ -158,8 +160,51 @@ class Package:
 		self.usePackageRoot = usePackageRoot
 		self.download       = download
 		self.location       = os.getcwd()
+		self.requires       = reqs
+		self.systemSetenv   = sys_setenv
+			
+		self.pubEnviros = []
+		for en in pubEnviros:
+			if os.environ.has_key(en): self.pubEnviros.append([en,os.environ[en]])
+			else:                      self.pubEnviros.append([en,''])
+
 		self.root           = ''
+#		print self.name,self.uninstall_coms
+#		if yesno('go on'): pass
 		
+	def reqs(self):
+		d = dir(self)
+		if phas(d,'requires'): return self.requires
+		else:                  return []
+		
+	def systemEnv(self):
+		d = dir(self)
+		if phas(d,'systemSetenv'): return self.systemSetenv
+		else:                      return ''
+	
+	def getPubEnviros(self):
+		d = dir(self)
+		if phas(d,'pubEnviros'): return self.pubEnviros
+		else:                    return []
+	
+	def req_enforce(self,pdb):
+		requirements = self.reqs()
+		nmissing = 0
+		for r in requirements:
+			if pdb.has_key(pn(r).name):
+				print r,pn(r).name,pn(r).cache,pdb[pn(r).name].cache
+				if pn(r).cache==pdb[pn(r).name].cache:
+					if verbose: print 'Required package ['+r+'] is installed.  Continuing...'
+				else:
+					nmissing = nmissing + 1
+					print 'You must install ['+r+'] first before fetching ['+self.name+'].'
+			else:
+				nmissing = nmissing + 1
+				print 'You must install ['+r+'] first before fetching ['+self.name+'].'
+			
+			if nmissing>0:
+				sys.exit()
+	
 	def fileRootUrl(self):
 		if not ( self.download=={} ):
 			if self.platform().has_key(findPlatform()):
@@ -250,6 +295,7 @@ class Package:
 		   self.setup            == p.setup          and \
 		   self.demo             == p.demo           and \
 		   self.suffixHandling   == p.suffixHandling and \
+		   self.systemEnv()      == p.systemEnv()    and \
 		   self.downloadEquiv(p) and \
 		   self.alreadyInstalled == p.alreadyInstalled:
 			return 1
@@ -294,7 +340,9 @@ class Package:
 		print ' natively installed conditions:  ',self.alreadyInstalled
 		print '              suffix handling?:  ',self.suffixHandling
 		print '   use package root directory?:  ',q_usePackageRoot(self)
+		print '               set system Env?:  ',self.systemEnv()
 		print '                      location:  ',self.location
+		print '  public environment variables:  ',`self.getPubEnviros()`
 		
 	def fetch(self,pdb,pg):
 		if self.fetched:
@@ -309,6 +357,8 @@ class Package:
 			self.fetched = 1
 			self.fetchtime = time.time()
 		else:
+			self.req_enforce(pdb)
+			
 			for pac in self.depends:
 				p = pg.get(pac)
 				p.fetch(pdb,pg)
@@ -317,7 +367,8 @@ class Package:
 #			if not self.download=={}: root = downloadroot(file)
 				
 			if not file=='':
-				print 'Fetching ['+url+']...'
+#				print 'Fetching ['+url+']...'
+				print 'Fetching ['+self.name+'] from ['+self.cache+']...'
 				try:
 					urllib.urlretrieve(url,file)
 				except IOError:
@@ -334,6 +385,7 @@ class Package:
 				self.root = ''
 			else:
 				self.root = root
+
 		pdb[self.name] = self
 		return pdb
 
@@ -343,7 +395,7 @@ class Package:
 			sys.exit()
 		elif self.installed: 
 #			pass
-			print 'Package ['+self.name+'] is already installed...'
+			if verbose: print 'Package ['+self.name+'] is already installed...'
 			self.set_enviros(self.root)
 			self.add_to_paths(self.root)
 		elif isAlreadyInstalled(self.alreadyInstalled):
@@ -357,7 +409,6 @@ class Package:
 					print "Can't install ["+pn(pac).name+"].  It hasn't been fetched."
 					sys.exit()
 			
-			print '*** ',self.location,self.root
 			self.set_enviros2 (self.location,self.root)
 			self.add_to_paths2(self.location,self.root)
 					
@@ -369,6 +420,12 @@ class Package:
 				else: 
 					print "Can't install ["+pn(pac).name+"].  It hasn't been fetched."
 					sys.exit()
+					
+			if not self.systemEnv()=='':
+				print 'Setting ['+self.systemEnv()+'] to point to this installation for all users...'
+				sysEnv = SystemEnviros()
+				sysEnv.insert(self.systemEnv())
+				sysEnv.place()
 
 			file,root,url = self.fileRootUrl()
 			if not self.download=={}: root = downloadroot(file)
@@ -406,18 +463,20 @@ class Package:
 #			if not self.source=='':
 			if not (self.systems=={} and self.download=={}):
 				if self.suffixHandling:
-					if tail(file,'.tar.gz') or tail(file,'.tgz'):
+					if tail(file,'.tar.gz') or tail(file,'.tgz') or tail(file,'.tar.Z'):
 #						if verbose: install_execute(self.name,'tar zxvPf '+file)
 #						else:       install_execute(self.name,'tar zxPf '+file)
 						if verbose: install_execute(self.name,'gunzip --stdout '+file+' | tar xvPf - ')
 						else:       install_execute(self.name,'gunzip --stdout '+file+' | tar xPf  - ')
-						if not string.strip(root)=='': self.uninstall_coms.insert(0,'rm -r -f '+root)
+						if not string.strip(root)=='' and not string.strip(root)=='.': 
+							self.uninstall_coms.insert(0,'rm -r -f '+root)
 					elif tail(file,'.tar'):
 						if verbose: install_execute(self.name,'tar xvPf '+file)
 						else:       install_execute(self.name,'tar xPf '+file)
-						if not string.strip(root)=='': self.uninstall_coms.insert(0,'rm -r -f '+root)
+						if not string.strip(root)=='' and not string.strip(root)=='.': 
+							self.uninstall_coms.insert(0,'rm -r -f '+root)
 					elif tail(file,'.rpm'):
-						if (not switch('no-rpmcheck')): install_execute(self.name,'rpm --checksig '+file)
+						if (switch('rpmcheck')): install_execute(self.name,'rpm --checksig '+file)
 						if rpm_replace(self.name):
 							install_execute(self.name,'rpm -U --force '+file)
 #							install_execute(self.name,'rm -f '+file)
@@ -429,12 +488,12 @@ class Package:
 					elif string.strip(file)=='':
 						pass
 					else:
-						print 'Downloaded file ['+file+'] must be .tar.gz, .tgz, .tar or .rpm.'
+						print 'Downloaded file ['+file+'] must be .tar.gz, .tgz, .tar.Z, .tar or .rpm.'
 						sys.exit()
 			
 			self.set_enviros(root)
 			self.add_to_paths(root)
-			if q_usePackageRoot(self) and not (self.systems=={} and self.download=={}):
+			if q_usePackageRoot(self) and not (self.systems=={} and self.download=={}) and self.suffixHandling:
 #				for command in commands: install_execute(self.name,'cd '+root+'; '+command)
 				for command in commands: 
 					install_execute2(self.name,root,os.path.expandvars(os.path.expanduser(shx(command))))
@@ -458,6 +517,11 @@ class Package:
 				for pac in self.depends:
 					if pdb.has_key(pn(pac).name): pdb[pn(pac).name].uninstall(pdb)
 				
+			if not self.systemEnv()=='':
+				sysEnv = SystemEnviros()
+				sysEnv.remove(self.systemEnv())
+				sysEnv.place()
+				
 			if self.location==os.getcwd(): 
 				ucoms = self.uninstall_coms
 				ucoms.reverse()
